/*
 *=BEGIN GPL
 *
 * This file is part of the PhotoResizerQt program.
 *
 * Copyright(c) 2010 Darrell Duffy
 * http://www.windyweather.net/wp
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 2 (the ``GPL'').
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the GPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *=END GPL
 */

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "aboutdialog.h"
#include "photoopendialog.h"

#include <QFileDialog>
#include <QListWidgetItem>
#include <QMessageBox>
#include <QApplication>


// these are missing from the library, or I don't know
// which library to include.

template <class T> const T& max ( const T& a, const T& b ) {
  return (b<a)?a:b;     // or: return comp(b,a)?a:b; for the comp version
}
template <class T> const T& min ( const T& a, const T& b ) {
  return (b>a)?a:b;     // or: return comp(b,a)?a:b; for the comp version
}

#if 0 // C version
int max( int one, int two )
{
    if ( one > two )    return one;
    else                return two;

}

int min( int one, int two )
{
    if ( one < two )    return one;
    else                return two;
}
#endif // C version

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    m_bPhotosResizing = false;
    m_bStopResizing = false;

    // set a library path to our local plugins
    QString appPath = qApp->applicationDirPath() +QString("/plugins");
    qApp->addLibraryPath( appPath );


#if 0
    // debug setting up the plugins path
    QStringList libPaths;
    libPaths = qApp->libraryPaths();
    for ( int i=0; i< libPaths.count(); i++ )
    {
        QString str = libPaths[i];
        ui->lwPhotoList->insertItem( i, str );

    }
#endif
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_actionQuit_triggered()
{
    qApp->exit(); // just exit the application
}

void MainWindow::on_actionAbout_triggered()
{
    if ( m_bPhotosResizing )
    {   return;
    }
    AboutDialog dialog(this);
    dialog.exec();              // show the About box as a modal dialog to keep it simple
}

void MainWindow::on_btnClose_clicked()
{
    qApp->exit(); // just exit the application
}

void MainWindow::resizeEvent ( QResizeEvent * event )
{
    setStatus("");

    resizeImages();
}

void MainWindow::setStatus( QString sts )
{
    ui->statusBar->showMessage( sts );
}

// set the photo name labels
void MainWindow::setListPhotoName( QString path )
{
    // set the path above the image.
    QFileInfo fi;
    fi.setFile(path);
    ui->lbPhotoListName->setText( fi.fileName() );
}

void MainWindow::setOnePhotoName( QString path )
{
    // set the path above the image.
    QFileInfo fi;
    fi.setFile(path);
    ui->lbOnePhotoName->setText( fi.fileName() );
}

// fix the images based on ShrinkToFit and size of window
void MainWindow::resizeImages()
{
    // if we have valid pixmap, then fix it
    if ( !m_onePixmap.isNull() )
    {
        QPixmap pixFit;

        if ( ui->cbShrinkPhotoPreview->isChecked() )
        {
            // If shrink, then shrink and set it
            pixFit = m_onePixmap.scaled(ui->gvPhotoPreview->width()-2,
                                        ui->gvPhotoPreview->height()-2, Qt::KeepAspectRatio );
        }
        else
        {
            pixFit = m_onePixmap;
        }

        ui->gvPhotoPreview->scene()->clear();
        ui->gvPhotoPreview->scene()->addPixmap( pixFit );

        // reset the scene scroll bars to the current image size
        ui->gvPhotoPreview->setSceneRect( ui->gvPhotoPreview->scene()->itemsBoundingRect() );
    }

    if ( !m_listPixmap.isNull() )
    {
        QPixmap pixFit;

        if ( ui->cbShrinkListPreview->isChecked() )
        {
            // If shrink, then shrink and set it
            pixFit = m_listPixmap.scaled(ui->gvPhotoListPreview->width()-2,
                                        ui->gvPhotoListPreview->height()-2, Qt::KeepAspectRatio );
        }
        else
        {
            pixFit = m_listPixmap;
        }

        ui->gvPhotoListPreview->scene()->clear();
        ui->gvPhotoListPreview->scene()->addPixmap( pixFit );

        // reset the scene scroll bars to the current image size
        ui->gvPhotoListPreview->setSceneRect( ui->gvPhotoListPreview->scene()->itemsBoundingRect() );
    }
}

// deal with image paths that are already resized
QStringList MainWindow::removeResizedImages( QStringList inList )
{
    int cnt = inList.count();
    QStringList outList;

    bool    bAnyResized = false;

    for ( int i=0; i < cnt; i++ )
    {
        if ( isResizedImage( inList[i]) )
        {
            // ignore it?
            bAnyResized = true;
        }
        else
        {
            outList.append( inList[i] );
        }
    }

    if ( bAnyResized )
    {
#if 0
        QMessageBox msgBox;
        msgBox.setText( tr("Some Images Cannot Be Resized.") );
        msgBox.setInformativeText(tr("Some resized images have been removed from the list.") );
        msgBox.setStandardButtons(QMessageBox::Ok );
        msgBox.setDefaultButton(QMessageBox::Ok);
        msgBox.setIcon(QMessageBox::Warning);
        msgBox.exec();
#endif
        setStatus( tr("Some resized images have been removed from the list."));
    }
    return outList;
}

// Does the image path contain xnnnn. ? then it's a resized image.
bool MainWindow::isResizedImage( QString path )
{
    QString foo;

    // filenamexnnnn.jpg
    QRegExp re(QString("x{1,1}[0-9]{1,4}\\.{1,1}"), Qt::CaseInsensitive );

    return path.contains( re );
}

// select a single photo to resize
void MainWindow::on_btnChoosePhoto_clicked()
{
    if ( m_bPhotosResizing )
    {   return;
    }
    setStatus("");

    PhotoOpenDialog dialog(this);

    dialog.setTitle(tr("Choose a Photo"));
    dialog.setSelectionMode( QAbstractItemView::SingleSelection );

    QStringList fileNames;
    if (dialog.exec())
    {
        // no path for the photo incase we get a bad one.
        m_onePhotoPath = "";

        fileNames = dialog.selectedFiles();
        QString path = fileNames[0];

        if ( this->isResizedImage( path ) )
        {
#if 0
            QMessageBox msgBox;
            msgBox.setText( tr("This Image Has Been Resized.") );
            msgBox.setInformativeText(tr("Select an image that has not been resized.") );
            msgBox.setStandardButtons(QMessageBox::Ok );
            msgBox.setDefaultButton(QMessageBox::Ok);
            msgBox.setIcon(QMessageBox::Warning);
            msgBox.exec();
#endif
            setStatus(tr("Select an image that has not been resized."));
            path = "";
            ui->lbOnePhotoName->setText("");
        }
        // load the image from the source path

         QGraphicsScene  *pScene = new QGraphicsScene(); // empty scene

         // add empty scene to the view.
         ui->gvPhotoPreview->setScene( pScene );
         ui->gvPhotoPreview->setDragMode( QGraphicsView::ScrollHandDrag );

         // if it's a valid image, then load into the scene.
         if ( m_onePixmap.load( path ) )
         {
             m_onePhotoPath = path;
             setOnePhotoName( path );

             setStatus(tr("Image read. Size (%1,%2)").arg(m_onePixmap.width()).arg(m_onePixmap.height()));

             // Resize the preview images based on settings.
             resizeImages();
         }
    }

}



// resize a single photo
void MainWindow::on_btnResizePhoto_clicked()
{
    if ( m_bPhotosResizing )
    {   return;
    }
    setStatus("");

    // if we have a photo open, then we can resize it.
    if ( ! m_onePixmap.isNull() )
    {
        // resize to the long-side as specified in the combo-box
        int nSize = ui->cbSize->currentText().toInt();

        // do not increase the size of any image.
        int nLongSide = max( m_onePixmap.height(), m_onePixmap.width() );
        if ( nLongSide < nSize )
        {
            setStatus(tr("Image is %1, which is smaller than the size requested. Not resized.").arg(nLongSide));
            return;
        }

        // make the resized pixmap. Keep Aspect and use best resize method.
        int newH = min( nSize, m_onePixmap.height() );
        int newW = min( nSize, m_onePixmap.width() );
        QPixmap newPix = m_onePixmap.scaled ( newW, newH, Qt::KeepAspectRatio, Qt::SmoothTransformation );

        // there might be some error.
        if ( newPix.isNull() )
        {
            setStatus(tr("Error resizing the image."));
            return;
        }
        // create a new path for the photo by adding xnnn from the size desired.
        // insert is destructive, so do it on the copy.
        QString newPath = m_onePhotoPath;
        int idx = newPath.lastIndexOf( QChar('.'));
        newPath.insert(idx, QString("x%1").arg(nSize));

        // if the new file exists, ask before we replace it.
        if ( QFile(newPath).exists() )
        {
            //setStatus(tr("Resized image exists."));
            QMessageBox msgBox;
            msgBox.setText( tr("This Image Has Been Resized to the selected size.") );
            msgBox.setInformativeText(tr("Resize and replace it?") );
            msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Cancel );
            msgBox.setDefaultButton(QMessageBox::Cancel);
            msgBox.setIcon(QMessageBox::Warning);
            int ret = msgBox.exec();
            if ( ret != QMessageBox::Save )
            {
                setStatus(tr("Image not resized."));
                return;
            }
        }
        // Assume something typical if we have an error.
        if ( !newPix.save( newPath ) )
        {
            setStatus(tr("Error saving image. Is it readonly, or on a CD-ROM?"));
        }
        else
        {
            // report the new size and the new path to confirm that we did something.
            setStatus(tr("Image resized to: (%1,%2) %3")
                      .arg(newPix.width()).arg(newPix.height()).arg(newPath) );
        }
    }
    else
    {
        setStatus(tr("Open an image first."));
    }

}

// Choose a bunch of photos to resize
void MainWindow::on_btnChoosePhotoList_clicked()
{
    if ( m_bPhotosResizing )
    {   return;
    }
    setStatus("");

    PhotoOpenDialog dialog(this);

    dialog.setTitle(tr("Choose Several Photos"));
    dialog.setSelectionMode( QAbstractItemView::ExtendedSelection );

    QStringList fileNames;
    if (dialog.exec())
    {
        fileNames = dialog.selectedFiles();
        if ( fileNames.count())
        {
            // remove any images from the list that are already resized.
            fileNames = removeResizedImages( fileNames );
            qDebug("on_btnChoosePhotoList_clicked - about to clear");

            //ui->lwPhotoList->clear(); // this crashes.
            // ASSERT failure in QList<T>::operator[]: "index out of range",
            // file ..\..\Qt\2010.05\qt\include/QtCore/../../src/corelib/tools/qlist.h, line 463
            // this crash also occurs if we remove the items one at a time, so
            // the list widget can only have zero items the first time, after that
            // if you remove all items, you get the above crash.
            // get around crash with more complex code

            int nitems = ui->lwPhotoList->count();
            qDebug("on_btnChoosePhotoList_clicked - Items %d", nitems);
            // take all the items out except one item.
            for ( int i=0; i<nitems-1; i++ )
            {
                qDebug("on_btnChoosePhotoList_clicked - take item %d", i);
                QListWidgetItem* item = ui->lwPhotoList->takeItem( 0 );
                qDebug("on_btnChoosePhotoList_clicked - delete item %d", i);
                delete item;
            }
            qDebug("on_btnChoosePhotoList_clicked - removed all items");

            // we are leaving 1 item in the list at all times.
            // so if first time there are no items, then put an empty item in the list now
            if ( nitems == 0 )
            {
                ui->lwPhotoList->insertItem(0, QString("") );
            }

            qDebug("on_btnChoosePhotoList_clicked - Clear, about to insert.");

            // avoid crash by adding items differently
#if 1 // try a simple way to do this
            QListWidgetItem* pitem = ui->lwPhotoList->item( 0 );
            pitem->setText( fileNames[0] );
            fileNames.removeFirst();
            // if items are left, insert them
            if ( fileNames.count() )
            {
                ui->lwPhotoList->insertItems(0, fileNames );
            }
#else // a more complicated way

            int nfnames = fileNames.count();
            qDebug("on_btnChoosePhotoList_clicked - inserting %d items", nfnames );
            for ( int i=0; i < nfnames; i++ )
            {
                QString fname;
                fname = fileNames[i];
                //qDebug("on_btnChoosePhotoList_clicked - chars in item %d", fname.count() );
                qDebug("on_btnChoosePhotoList_clicked - filename[%d] %d chars", i, fname.count());
                // we left an item, so change it's text string
                if ( i == 0 )
                {
                    QListWidgetItem* pitem = ui->lwPhotoList->item( 0 );
                    pitem->setText( fname );

                }
                else
                {
                    ui->lwPhotoList->insertItem( i, fname );
                }
            }
#endif // more complicated way
            qDebug("on_btnChoosePhotoList_clicked - items inserted");

            QGraphicsScene  *pScene = new QGraphicsScene(); // empty scene

            // add empty scene to the view.
            ui->gvPhotoListPreview->setScene( pScene );
            ui->gvPhotoListPreview->setDragMode( QGraphicsView::ScrollHandDrag );

            // sort the file list now
            ui->lwPhotoList->sortItems();

            // if it's a valid image, then load the first image into the scene.
            QString path = ui->lwPhotoList->item(0)->text();
            ui->lwPhotoList->setCurrentRow(0);
            // set the path above the image.
            setListPhotoName( path );
            if ( m_listPixmap.load( path ) )
            {
                // Display the image if we got one
                resizeImages();
            }
        }
    }
}

// Resize a single photo given the path and the target size
// return true if we should continue.
bool MainWindow::resizeOnePhoto( QString path, int nSize )
{

    // load the image from the source path
     // if it's a valid image, then load into the scene.
     if ( m_listPixmap.load( path ) )
     {
         setListPhotoName( path );
         // Resize the images based on settings.
         resizeImages();
     }
     else
     {
         // some error stops us
         setStatus(tr("Error reading image: %1").arg(path));
         return false;
     }
    // do not increase the size of any image.
    int nLongSide = max( m_listPixmap.height(), m_listPixmap.width() );
    if ( nLongSide < nSize )
    {
        setStatus(tr("Image is %1, which is smaller than the size requested. Not resized.").arg(nLongSide));
        return true;
    }

    // make the resized pixmap. Keep Aspect and use best resize method.
    int newH = min( nSize, m_listPixmap.height() );
    int newW = min( nSize, m_listPixmap.width() );
    QPixmap newPix = m_listPixmap.scaled ( newW, newH, Qt::KeepAspectRatio, Qt::SmoothTransformation );

    // there might be some error.
    if ( newPix.isNull() )
    {
        setStatus(tr("Error resizing the image: %1").arg(path));
        return false;
    }
    // create a new path for the photo by adding xnnn from the size desired.
    // insert is destructive, so do it on the copy.
    QString newPath = path;
    int idx = newPath.lastIndexOf( QChar('.'));
    newPath.insert(idx, QString("x%1").arg(nSize));

    // if the new file exists, ask before we replace it.
    if ( QFile(newPath).exists() )
    {
        //setStatus(tr("Resized image exists."));
        QMessageBox msgBox;
        msgBox.setText( tr("The Image Has Been Resized to the selected size.") );
        msgBox.setInformativeText(tr("Resize and replace?\n%1").arg(newPath) );
        msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Cancel );
        msgBox.setDefaultButton(QMessageBox::Cancel);
        msgBox.setIcon(QMessageBox::Warning);
        int ret = msgBox.exec();
        if ( ret != QMessageBox::Save )
        {
            setStatus(tr("Image not resized."));
            return false;
        }
    }
    // Assume something typical if we have an error.
    if ( !newPix.save( newPath ) )
    {
        setStatus(tr("Error saving image. Is it readonly, or on a CD-ROM?"));
        return false;
    }
    else
    {
        // report the new size and the new path to confirm that we did something.
        setStatus(tr("Image resized to: (%1,%2) %3")
                  .arg(newPix.width()).arg(newPix.height()).arg(newPath) );
    }
    return true;
}


// resize all the images in the list to the save size
void MainWindow::on_btnResizePhotoList_clicked()
{
    if ( m_bPhotosResizing )
    {   return;
    }
    setStatus("");

    // resize to the long-side as specified in the combo-box
    int nSize = ui->cbSizeList->currentText().toInt();

    int items = ui->lwPhotoList->count();
    if ( items == 0 )
    {
        setStatus(tr("Open some photos first."));
        return;
    }
    // set control data
    m_bPhotosResizing = true;
    m_bStopResizing = false;
    int nItemsResized = 0;

    QListWidgetItem*    item;
    QEventLoop          eventloop;
    bool    bOk = true;

    // walk through the items and do each one in turn.
    ui->pbResizeList->setMinimum(0);
    ui->pbResizeList->setMaximum( items );
    ui->pbResizeList->setValue( 0 );
    for ( int i=0; i < items; i++ )
    {
        item = ui->lwPhotoList->item(i);
        bOk = resizeOnePhoto( item->text(), nSize );
        if ( !bOk )
        {
            break;
        }

        nItemsResized++;
        ui->pbResizeList->setValue( i+1 );

        // Process events to allow image to update
        // and allow us to cancel the resize.
        eventloop.processEvents();
        if ( m_bStopResizing )
        {   break;
        }
    }
    if ( bOk && nItemsResized == items )
    {
        setStatus(tr("All images resized successfully."));
    }
    else
    {
        setStatus(tr("%1 Images resized").arg(nItemsResized) );
    }
    ui->pbResizeList->setValue( items );

    // set control data
    m_bPhotosResizing = false;
    m_bStopResizing = false;
}

// select a photo to preview
void MainWindow::on_lwPhotoList_itemClicked(QListWidgetItem* item)
{
    if ( m_bPhotosResizing )
    {   return;
    }
    setStatus("");

    QString path = item->text();
    setListPhotoName( path );

    // load the image from the source path
     // if it's a valid image, then load into the scene.
     if ( m_listPixmap.load( path ) )
     {
         setStatus(tr("Previewing: %1").arg(path));
         // Resize the images based on settings.
         resizeImages();
     }
}

// when the selection changes then change the preview
void MainWindow::on_lwPhotoList_itemSelectionChanged()
{
    if ( m_bPhotosResizing )
    {   return;
    }
    QList<QListWidgetItem*> items = ui->lwPhotoList->selectedItems();

    on_lwPhotoList_itemClicked( items[0] );
}

void MainWindow::on_splitter_splitterMoved(int pos, int index)
{
    setStatus("");

   resizeImages();
}

void MainWindow::on_cbShrinkListPreview_stateChanged(int )
{
    setStatus("");

    resizeImages();
}

void MainWindow::on_cbShrinkPhotoPreview_stateChanged(int )
{
    setStatus("");

    resizeImages();
}

// click cancel to stop resizing a long list
void MainWindow::on_btnCancel_clicked()
{
    if ( m_bPhotosResizing )
    {   m_bStopResizing = true;
    }
}

void MainWindow::on_tabWidget_currentChanged(int index)
{
    resizeImages();
}
