/*
 *=BEGIN GPL
 *
 * This file is part of the PhotoResizerQt program.
 *
 * Copyright(c) 2010 Darrell Duffy
 * http://www.windyweather.net/wp
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 2 (the ``GPL'').
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the GPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *=END GPL
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QPixmap>

class QListWidgetItem;

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    Ui::MainWindow *ui;

    // Pixmaps for preview images
    QPixmap m_onePixmap;
    QPixmap m_listPixmap;

    // path of a single photo, saved to resize it later
    QString     m_onePhotoPath;

    bool        m_bPhotosResizing;
    bool        m_bStopResizing;

    void resizeEvent ( QResizeEvent * event );

    // resize images when we resize the dialog
    void resizeImages();

    // deal with image paths that are already resized
    QStringList removeResizedImages( QStringList inList );
    bool        isResizedImage( QString path );

    // set the status line to a message.
    void setStatus( QString sts );

    // resize a single photo
    bool resizeOnePhoto( QString path, int nSize );

    // set the photo name labels
    void setListPhotoName( QString path );
    void setOnePhotoName( QString path );

private slots:
    void on_tabWidget_currentChanged(int index);
    void on_btnCancel_clicked();
    void on_lwPhotoList_itemSelectionChanged();
    void on_cbShrinkPhotoPreview_stateChanged(int );
    void on_cbShrinkListPreview_stateChanged(int );
    void on_splitter_splitterMoved(int pos, int index);
    void on_lwPhotoList_itemClicked(QListWidgetItem* item);
    void on_btnResizePhotoList_clicked();
    void on_btnChoosePhotoList_clicked();
    void on_btnResizePhoto_clicked();
    void on_btnChoosePhoto_clicked();
    void on_btnClose_clicked();
    void on_actionAbout_triggered();
    void on_actionQuit_triggered();
};

#endif // MAINWINDOW_H
