/*
 *=BEGIN GPL
 *
 * This file is part of the PhotoOpenDialog program.
 *
 * Copyright (c) 2011 Darrell Duffy
 * http://www.windyweather.net/wp
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 3 (the ``GPL'').
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the GPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *=END GPL
 */

#ifndef PHOTOOPENDIALOG_H
#define PHOTOOPENDIALOG_H

#include <QDialog>
#include <QSettings>
#include <QAbstractItemView>

class QTreeWidgetItem;

namespace Ui {
    class PhotoOpenDialog;
}

class PhotoOpenDialog : public QDialog
{
    Q_OBJECT

public:
    explicit PhotoOpenDialog(QWidget *parent = 0);
    ~PhotoOpenDialog();

    // remove resized image from the list.
    bool isResizedImage( QString path );

    // set parameters from the dialog
    void setTitle( QString title );

    /*
QAbstractItemView::SingleSelection	1	When the user selects an item, any already-selected item becomes unselected, and the user cannot unselect the selected item by clicking on it.
QAbstractItemView::ContiguousSelection	4	When the user selects an item in the usual way, the selection is cleared and the new item selected. However, if the user presses the Shift key while clicking on an item, all items between the current item and the clicked item are selected or unselected, depending on the state of the clicked item.
QAbstractItemView::ExtendedSelection	3	When the user selects an item in the usual way, the selection is cleared and the new item selected. However, if the user presses the Ctrl key when clicking on an item, the clicked item gets toggled and all other items are left untouched. If the user presses the Shift key while clicking on an item, all items between the current item and the clicked item are selected or unselected, depending on the state of the clicked item. Multiple items can be selected by dragging the mouse over them.
QAbstractItemView::MultiSelection	2	When the user selects an item in the usual way, the selection status of that item is toggled and the other items are left alone. Multiple items can be toggled by dragging the mouse over them.
QAbstractItemView::NoSelection          0	Items cannot be selected.
     */
    void setSelectionMode( QAbstractItemView::SelectionMode mode);

    // Return the list of paths to the selected photos
    QStringList selectedFiles();

private:
    Ui::PhotoOpenDialog *ui;
    QSettings*          m_pSettings;
    int                 m_nIconSize;    // size of icons in the folder tree

    // Allow the user to stop processing a folder by hitting cancel.
    // Cancel will not close the dialog in this case but will just stop processing the folder.
    bool                m_bProcessingFolder;        // true if we are processing a folder
    bool                m_bStopProcessingFolder;    // set to true by Cancel if we should stop.


    void setStatus( QString sts );

    // fixup a path and insert it in the folder list
    QString fixAndInsertPath( QString path );
    QString fixFolderPath( QString path );

    // Save the list of folders we have seen.
    void createSettings();
    void saveSettings();
    void restoreSettings();

    // save stuff and treat like a cancel - sort of
    void closeEvent ( QCloseEvent * event );

private slots:
    void on_btnUpArrow_clicked();
    void on_twFolder_itemActivated(QTreeWidgetItem* item, int column);
    void on_cbFolderPath_activated(QString );
    void on_cbIconSize_activated(QString );
    void on_btnBrowseFolder_clicked();
    void on_btnOpen_clicked();
    void on_btnCancel_clicked();
};

#endif // PHOTOOPENDIALOG_H
